@@ -22,8 +22,8 @@ import siputils
 
 
 # Initialise the globals.
-sip_version = 0x030a01
-sip_version_str = "3.10.1"
+sip_version = 0x030a02
+sip_version_str = "3.10.2"
 qt_version = 0
 qt_edition = ""
 qt_winconfig = ""
@@ -31,7 +31,6 @@ qt_threaded = 0
 qt_dir = None
 qt_incdir = None
 qt_libdir = None
-qt_prl_libs = None
 py_version = sys.hexversion >> 8
 plat_py_site_dir = None
 plat_py_inc_dir = None
@@ -104,7 +103,7 @@ def usage(rcode = 2):
     print "    -d dir         where the SIP module will be installed [default %s]" % opt_sipmoddir
     print "    -e dir         where the SIP header file will be installed [default %s]" % opt_sipincdir
     print "    -k             build the SIP module as a static library"
-    print "    -l Qt-library  explicitly specify the type of Qt library, either qt, qt-mt, qt-mtedu, qte, qtmt or qtmtedu"
+    print "    -l Qt-library  explicitly specify the type of Qt library, either qt, qt-mt, qte or qtmt"
     print "    -p platform    the platform/compiler configuration [default the Qt configuration if enabled or %s]" % default_platform
     print "    -u             build with debugging symbols"
     print "    -v dir         where .sip files are normally installed [default %s]" % opt_sipsipdir
@@ -285,7 +284,6 @@ def create_config(module, template, macros):
         content["qt_inc_dir"] = qt_incdir
         content["qt_lib"] = opt_qtlib
         content["qt_lib_dir"] = qt_libdir
-        content["qt_prl_libs"] = qt_prl_libs
 
     # Write the (nicely formatted) dictionary.
     content = siputils.create_content(content) + "_default_macros = {\n"
@@ -318,6 +316,12 @@ def check_qt_installation(macros):
 
     macros is the dictionary of macros.
     """
+    # Get the Makefile generator.
+    try:
+        generator = macros["MAKEFILE_GENERATOR"]
+    except KeyError:
+        generator = "UNIX"
+
     # Check the Qt header files have been installed.
     global qt_incdir
 
@@ -376,17 +380,18 @@ def check_qt_installation(macros):
         qt_libdir = os.path.join(qt_dir, "lib")
 
     if opt_qtlib:
-        if not is_qt_library(opt_qtlib):
+        if not is_qt_library(generator, opt_qtlib):
             siputils.error("The %s Qt library could not be found in %s." % (opt_qtlib, qt_libdir))
     else:
-        stlib = is_qt_library("qt")
-        mtlib = is_qt_library("qt-mt")
-        edlib = is_qt_library("qt-mtedu")
-        emlib = is_qt_library("qte")
+        stlib = is_qt_library(generator, "qt")
+        mtlib = is_qt_library(generator, "qt-mt")
+        edlib = is_qt_library(generator, "qt-mtedu")
+        evlib = is_qt_library(generator, "qt-mteval")
+        emlib = is_qt_library(generator, "qte")
 
         # Borland likes to be a little different.
-        bmtlib = is_qt_library("qtmt")
-        bedlib = is_qt_library("qtmtedu")
+        bmtlib = is_qt_library(generator, "qtmt")
+        bedlib = is_qt_library(generator, "qtmtedu")
 
         names = []
 
@@ -402,6 +407,10 @@ def check_qt_installation(macros):
             opt_qtlib = "qt-mtedu"
             names.append(opt_qtlib)
 
+        if evlib:
+            opt_qtlib = "qt-mteval"
+            names.append(opt_qtlib)
+
         if emlib:
             opt_qtlib = "qte"
             names.append(opt_qtlib)
@@ -421,40 +430,10 @@ def check_qt_installation(macros):
             siputils.error("These Qt libraries were found: %s. Use the -l argument to explicitly specify which you want to use." % string.join(names))
 
     # See if Qt has thread support.
-    if opt_qtlib in ("qt-mt", "qt-mtedu", "qtmt", "qtmtedu"):
+    if opt_qtlib in ("qt-mt", "qt-mtedu", "qt-mteval", "qtmt", "qtmtedu"):
         global qt_threaded
         qt_threaded = 1
 
-    # Try and get the additional libraries that Qt is linked against from the
-    # .prl file.
-    if sys.platform == "win32":
-        prl_name = os.path.join(qt_libdir, opt_qtlib + ".prl")
-    else:
-        prl_name = os.path.join(qt_libdir, "lib" + opt_qtlib + ".prl")
-
-    if os.access(prl_name, os.F_OK):
-        global qt_prl_libs
-
-        qt_prl_libs = ""
-
-        try:
-            f = open(prl_name, "r")
-        except IOError:
-            siputils.error("Unable to open \"%s\": %s" % (prl_name, detail))
-
-        line = f.readline()
-        while line:
-            line = string.strip(line)
-            if line and line[0] != "#":
-                eq = string.find(line, "=")
-                if eq > 0 and string.strip(line[:eq]) == "QMAKE_PRL_LIBS":
-                    qt_prl_libs = string.strip(line[eq + 1:])
-                    break
-
-            line = f.readline()
-
-        f.close()
-
     # Try and work out how Qt was built on Windows.
     if sys.platform == "win32":
         global qt_winconfig
@@ -477,12 +456,13 @@ def check_qt_installation(macros):
             qt_winconfig = "shared"
 
 
-def is_qt_library(lib):
+def is_qt_library(generator, lib):
     """See if a particular Qt library is installed.
 
+    generator is the name of the Makefile generator.
     lib is the name of the library.
     """
-    if sys.platform == "win32":
+    if generator != "UNIX" and generator != "MINGW":
         lpatts = [lib + "[0-9]*.lib", lib + ".lib"]
     else:
         lpatts = ["lib" + lib + ".*"]
@@ -783,7 +763,7 @@ def main(argv):
         elif opt == "-k":
             opt_static = 1
         elif opt == "-l":
-            if arg in ("qt", "qt-mt", "qt-mtedu", "qte", "qtmt", "qtmtedu"):
+            if arg in ("qt", "qt-mt", "qt-mtedu", "qt-mteval", "qte", "qtmt", "qtmtedu"):
                 opt_qtlib = arg
             else:
                 usage()
