@@ -218,6 +218,11 @@ class Makefile:
             if m in ("DEFINES", "INCDIR", "LIBDIR"):
                 val = string.split(val)
 
+            # We also want to treat lists of libraries in the same way so that
+            # duplicates get eliminated.
+            if m[:4] == "LIBS":
+                val = string.split(val)
+
             self.__dict__[m] = _Macro(m, val)
 
         # This is used to alter the configuration more significantly than can
@@ -262,16 +267,12 @@ class Makefile:
             win_exceptions = ("exceptions" in wcfg)
             win_rtti = ("rtti" in wcfg)
             win_stl = ("stl" in wcfg)
-
-            use_prl = (self.config.qt_prl_libs is not None)
         else:
             win_shared = 1
             win_exceptions = 0
             win_rtti = 0
             win_stl = 0
 
-            use_prl = 0
-
         # Get what we are going to transform.
         cflags = _UniqueList()
         cflags.extend(self.extra_cflags)
@@ -297,10 +298,11 @@ class Makefile:
         libdir.extend(self.extra_lib_dirs)
         libdir.extend(self.optional_list("LIBDIR"))
 
-        libs = []
+        libs = _UniqueList()
 
         for l in self.extra_libs:
             libs.append(self.platform_lib(l))
+            libs.extend(self._dependent_libs(l))
 
         libs.extend(self.optional_list("LIBS"))
 
@@ -396,12 +398,6 @@ class Makefile:
             cxxflags.extend(self.optional_list("CXXFLAGS_THREAD"))
             lflags.extend(self.optional_list("LFLAGS_THREAD"))
 
-        if self.console:
-            libs.extend(self.optional_list("LIBS_CONSOLE"))
-
-        if not use_prl:
-            libs.extend(self.optional_list("LIBS_WINDOWS"))
-
         if self._qt:
             if self.generator != "UNIX" and win_shared:
                 defines.append("QT_DLL")
@@ -418,10 +414,10 @@ class Makefile:
             libdir.extend(libdir_qt)
             rpaths.extend(libdir_qt)
 
-            # Windows needs the version number appended.
+            # Windows needs the version number appended if Qt is a DLL.
             qt_lib = self.config.qt_lib
 
-            if sys.platform == "win32" and win_shared:
+            if self.generator != "UNIX" and self.generator != "MINGW" and win_shared:
                 qt_lib = qt_lib + string.replace(version_to_string(self.config.qt_version), ".", "")
 
             libs.append(self.platform_lib(qt_lib))
@@ -429,23 +425,24 @@ class Makefile:
         if self._opengl:
             incdir.extend(self.optional_list("INCDIR_OPENGL"))
             lflags.extend(self.optional_list("LFLAGS_OPENGL"))
-
-            if not use_prl:
-                libdir.extend(self.optional_list("LIBDIR_OPENGL"))
-                libs.extend(self.optional_list("LIBS_OPENGL"))
+            libdir.extend(self.optional_list("LIBDIR_OPENGL"))
+            libs.extend(self.optional_list("LIBS_OPENGL"))
 
         if self._qt or self._opengl:
             incdir.extend(self.optional_list("INCDIR_X11"))
+            libdir.extend(self.optional_list("LIBDIR_X11"))
+            libs.extend(self.optional_list("LIBS_X11"))
 
-            if not use_prl:
-                libdir.extend(self.optional_list("LIBDIR_X11"))
-                libs.extend(self.optional_list("LIBS_X11"))
-
-        if self._threaded and not use_prl:
+        if self._threaded:
             libs.extend(self.optional_list("LIBS_THREAD"))
 
-        if use_prl:
-            libs.extend(string.split(self.config.qt_prl_libs))
+        if self.console:
+            libs.extend(self.optional_list("LIBS_CONSOLE"))
+
+        libs.extend(self.optional_list("LIBS_WINDOWS"))
+
+        if self._qt:
+            libs.extend(self._dependent_libs(self.config.qt_lib))
 
         lflags.extend(self._platform_rpaths(rpaths.as_list()))
 
@@ -510,15 +507,50 @@ class Makefile:
     def platform_lib(self, clib):
         """Return a library name in platform specific form.
 
-        clib is a library name in cannonical form.
+        clib is the library name in cannonical form.
         """
-        if self.generator != "UNIX":
+        if self.generator != "UNIX" and self.generator != "MINGW":
             plib = clib + ".lib"
         else:
             plib = "-l" + clib
 
         return plib
 
+    def _dependent_libs(self, clib):
+        """Return a list of additional libraries (in platform specific form)
+        that must be linked with a library.
+
+        clib is the library name in cannonical form.
+        """
+        prl_libs = []
+
+        if self.generator != "UNIX" and self.generator != "MINGW":
+            prl_name = os.path.join(self.config.qt_lib_dir, clib + ".prl")
+        else:
+            prl_name = os.path.join(self.config.qt_lib_dir, "lib" + clib + ".prl")
+
+        if os.access(prl_name, os.F_OK):
+            try:
+                f = open(prl_name, "r")
+            except IOError:
+                error("Unable to open \"%s\": %s" % (prl_name, detail))
+
+            line = f.readline()
+            while line:
+                line = string.strip(line)
+                if line and line[0] != "#":
+                    eq = string.find(line, "=")
+                    if eq > 0 and string.strip(line[:eq]) == "QMAKE_PRL_LIBS":
+                        prl_libs = string.split(line[eq + 1:])
+                        break
+
+                line = f.readline()
+
+            f.close()
+
+        return prl_libs
+
+
     def parse_build_file(self, filename):
         """
         Parse a build file and return the corresponding dictionary.
@@ -571,7 +603,7 @@ class Makefile:
                 dict[i] = ""
 
         # Generate the list of objects.
-        if self.generator != "UNIX":
+        if self.generator != "UNIX" and self.generator != "MINGW":
             ext = ".obj"
         else:
             ext = ".o"
@@ -669,7 +701,7 @@ class Makefile:
 
         libs = []
 
-        if self.generator != "UNIX":
+        if self.generator != "UNIX" and self.generator != "MINGW":
             libdir_prefix = "/LIBPATH:"
         else:
             libdir_prefix = "-L"
@@ -684,10 +716,16 @@ class Makefile:
         if self._qt:
             mfile.write("MOC = %s\n" % _quote(self.required_string("MOC")))
 
-        if self.generator != "UNIX":
-            mfile.write("""
-.SUFFIXES: .c .cpp .cc .cxx .C
+        # These probably don't matter.
+        if self.generator == "MINGW":
+            mfile.write(".SUFFIXES: .cpp .cxx .cc .C .c\n\n")
+        elif self.generator == "UNIX":
+            mfile.write(".SUFFIXES: .c .o .cpp .cc .cxx .C\n\n")
+        else:
+            mfile.write(".SUFFIXES: .c .cpp .cc .cxx .C\n\n")
 
+        if self.generator != "UNIX" and self.generator != "MINGW":
+            mfile.write("""
 {.}.cpp{}.obj::
 \t$(CXX) -c $(CXXFLAGS) $(CPPFLAGS) -Fo @<<
 \t$<
@@ -715,8 +753,6 @@ class Makefile:
 """)
         else:
             mfile.write("""
-.SUFFIXES: .c .o .cpp .cc .cxx .C
-
 .cpp.o:
 \t$(CXX) -c $(CXXFLAGS) $(CPPFLAGS) -o $@ $<
 
@@ -763,7 +799,8 @@ class Makefile:
         dst is the name of the destination directory.
         """
         # Help package builders.
-        dst = "$(DESTDIR)" + dst
+        if self.generator == "UNIX":
+            dst = "$(DESTDIR)" + dst
 
         mfile.write("\t@%s %s " % (self.chkdir, _quote(dst)))
 
@@ -835,7 +872,9 @@ class ParentMakefile(Makefile):
         mfile.write("\n" + target + ":\n")
 
         for d in self._subdirs:
-            if self.generator != "UNIX":
+            if self.generator == "MINGW":
+                mfile.write("\t@$(MAKE) -C %s%s\n" % (d, tname))
+            elif self.generator != "UNIX":
                 mfile.write("\tcd %s\n" % d)
                 mfile.write("\t$(MAKE)%s\n" % tname)
                 mfile.write("\t@cd ..\n")
@@ -914,7 +953,9 @@ class ModuleMakefile(Makefile):
         Makefile.finalise(self)
 
         if not self.static:
-            if self.optional_string("AIX_SHLIB"):
+            if self.generator == "MINGW" and self.config.sip_version < 0x040000:
+                self.LFLAGS.append("-Wl,--out-implib,%s.a" % self._target)
+            elif self.optional_string("AIX_SHLIB"):
                 # AIX needs a lot of special handling.
                 if self.optional_string("LINK_SHLIB_CMD"):
                     if self.config.sip_version >= 0x040000:
@@ -975,7 +1016,7 @@ class ModuleMakefile(Makefile):
             if sys.platform != "win32":
                 mname = mname + "module"
 
-        if sys.platform == "win32":
+        if self.generator != "UNIX" and self.generator != "MINGW":
             mname = "lib" + mname
 
         return mname
@@ -1004,7 +1045,7 @@ class ModuleMakefile(Makefile):
         mfile.write("\n")
 
         if self.static:
-            if self.generator != "UNIX":
+            if self.generator != "UNIX" and self.generator != "MINGW":
                 mfile.write("LIB = %s\n" % self.required_string("LIB"))
             else:
                 mfile.write("AR = %s\n" % self.required_string("AR"))
@@ -1023,7 +1064,7 @@ class ModuleMakefile(Makefile):
         """
         mfile.write("\n$(TARGET): $(OFILES)\n")
 
-        if self.generator != "UNIX":
+        if self.generator != "UNIX" and self.generator != "MINGW":
             if self.static:
                 mfile.write("\t$(LIB) /OUT:$(TARGET) @<<\n")
                 mfile.write("\t  $(OFILES)\n")
@@ -1062,9 +1103,12 @@ class ModuleMakefile(Makefile):
         mfile.write("\ninstall: $(TARGET)\n")
         self.install_file(mfile, "$(TARGET)", self._install_dir)
 
-        if self.generator != "UNIX" and self.config.sip_version < 0x040000 and not self.static:
-            self.install_file(mfile, self._target + ".lib", self._install_dir)
-            self.install_file(mfile, self._target + ".exp", self._install_dir)
+        if self.config.sip_version < 0x040000 and not self.static:
+            if self.generator == "MINGW":
+                self.install_file(mfile, self._target + ".a", self._install_dir)
+            elif self.generator != "UNIX":
+                self.install_file(mfile, self._target + ".lib", self._install_dir)
+                self.install_file(mfile, self._target + ".exp", self._install_dir)
 
     def generate_target_clean(self, mfile):
         """Generate the clean target.
@@ -1143,7 +1187,7 @@ class ProgramMakefile(Makefile):
 
         build.append(source)
 
-        if self.generator != "UNIX":
+        if self.generator != "UNIX" and self.generator != "MINGW":
             build.append("-Fe")
             build.append("/link")
             libdir_prefix = "/LIBPATH:"
@@ -1164,7 +1208,7 @@ class ProgramMakefile(Makefile):
     def finalise(self):
         """Finalise the macros for a program Makefile.
         """
-        if self.generator != "UNIX":
+        if self.generator != "UNIX" and self.generator != "MINGW":
             self.LFLAGS.append("/INCREMENTAL:NO")
 
         if self.console:
@@ -1203,7 +1247,7 @@ class ProgramMakefile(Makefile):
         """
         mfile.write("\n$(TARGET): $(OFILES)\n")
 
-        if self.generator != "UNIX":
+        if self.generator != "UNIX" and self.generator != "MINGW":
             mfile.write("\t$(LINK) $(LFLAGS) /OUT:$(TARGET) @<<\n")
             mfile.write("\t  $(OFILES) $(LIBS)\n")
             mfile.write("<<\n")
